==> nono-0.7.3.tar.gz: nono-0.7.3/vm/mpu680x0.h <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU (MC68030)
//

#pragma once

#include "mpu.h"
#include "branchhistory.h"
#include "bus.h"
#include "event.h"
#include "m68030.h"
#include "m68030mmu.h"
#include "mainbus.h"

class InterruptDevice;
class m68030Cache;

class MPU680x0Device : public MainMPUDevice
{
	using inherited = MainMPUDevice;
	friend class m68030MMU;

	// CPU の実行状態
	static const uint32 CPU_STATE_NORMAL	= 0;
	static const uint32 CPU_STATE_STOP		= 1;
	static const uint32 CPU_STATE_HALT		= 2;

 public:
	MPU680x0Device();
	~MPU680x0Device() override;

	void SetLogLevel(int loglevel_) override;
	bool Init() override;
	void ResetHard(bool poweron) override;

	// 現在の PPC を取得
	uint32 GetPPC() const override { return ppc; }

	// 現在の VBR を取得
	uint32 GetVBR() const override { return reg.vbr; }

	// MPU 名を取得
	const char *GetMPUName() const override { return "MC68030"; }

	bool IsSuper() const	{ return reg.s; }
	bool IsMaster() const	{ return reg.m; }

	// CPU の実行状態 (uint32) を返す。
	uint32 GetState() const	{ return cpu_state; }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override { return bus.laddr.Addr(); }
	uint32 GetPaddr() const override { return bus.paddr.Addr(); }

	// 割り込みレベルが変わったことを MPU に通知
	void Interrupt(int level) override;

	// A-Line, F-Line 命令エミュレーションのコールバックを指定する。
	// (Human68k エミュレーションで使用)
	void SetALineCallback(bool (*callback)(MPU680x0Device*, void*), void *arg);
	void SetFLineCallback(bool (*callback)(MPU680x0Device*, void*), void *arg);

	// FPU を持っているか
	bool HaveFPU() const { return has_fpu; }

	// バーストアクセスをサポートするか
	void EnableBurstAccess(bool enable) { has_burst_access = enable; }

	void SetCI() override {
		bus.ci = true;
	}

	// ホールト
	void Halt();

	// 基本アクセス。内部から使用する。
	// アドレスは論理アドレスで、空間は現在の命令/データ空間。
	// バスエラーなら C++ の例外をスローする。
	uint32 fetch_16();
	uint32 fetch_32();
	uint32 read_8(uint32 addr);
	uint32 read_16(uint32 addr);
	uint32 read_32(uint32 addr);
	void write_8(uint32 addr, uint32 data);
	void write_16(uint32 addr, uint32 data);
	void write_32(uint32 addr, uint32 data);

	// MMU アクセス
	busaddr TranslatePeek(busaddr laddr);
	// 統計
	void CalcATC();

	// レジスタ
	uint16 GetIR() const	{ return ir; }
	uint16 GetIR2() const	{ return ir2; }
	// SR を変更する (Human68k から使う)
	void SetSR(uint16);
	uint16 GetSR() const	{ return reg.GetSR(); }

	uint64 GetSRP()  const { return reg.srp.q; }
	uint32 GetSRPh() const { return reg.srp.h; }
	uint32 GetSRPl() const { return reg.srp.l; }
	uint64 GetCRP()  const { return reg.crp.q; }
	uint32 GetCRPh() const { return reg.crp.h; }
	uint32 GetCRPl() const { return reg.crp.l; }
	uint64 GetTT(int idx) const { return reg.tt[idx]; }
	uint32 GetTC() const { return reg.tc; }
	uint16 GetMMUSR() const { return reg.mmusr; }

 private:
	// リセット例外コールバック
	void ResetCallback(Event& ev);

	// 命令実行系コールバック
	void ExecShort(Event& ev);
	void ExecFull(Event& ev);
	void ExecTrace(Event& ev);

	// トレース設定。デバッガから呼び出される。
	void SetTrace(bool trace_on) override;

	// 割り込みイベントコールバック
	void InterruptCallback(Event& ev);

	// 動作モード変更
	void ChangeState(uint32 new_state);

	// 次の実行サイクルを Full にする。
	void MakeNextFull();

	inline void CYCLE(int cc) {
		used_cycle += cc;
	}
	inline void CYCLE2(int cc, int ncc) {
		used_cycle += cc;
		cycle_nocache += ncc - cc;
	}

	// PC を addr に変更する。
	// ベクタへのジャンプはここではなく JumpVector()。
	inline void Jump(uint32 addr) {
		// 先にブランチ履歴に記録
		brhist.AddEntry(ppc | (IsSuper() ? 1 : 0), addr, ir);

		if (__predict_false((addr & 1) != 0)) {
			ExceptionBuserr(M68K::EXCEP_ADDRERR);
			return;
		}
		reg.pc = addr;
	}

	// -(An) 実行前にレジスタを保存する。
	// バスエラーからの復帰用。
	inline void save_reg_pd(int n) {
		// 該当ビットのフラグを立てる
		flag_pd |= (1 << n);

		// 保存
		save_pd[n] = reg.A[n];
	}

	// (An)+ 実行前にレジスタを保存する。
	// バスエラーからの復帰用。
	inline void save_reg_pi(int n) {
		// 該当ビットのフラグを立てる
		flag_pi |= (1 << n);

		// 保存
		save_pi[n] = reg.A[n];
	}

	void restore_reg_pi();
	void restore_reg_pd();
	void JumpVector(uint vector, bool from_buserr);

 protected:
#include "m68030ea.h"

 private:
	// FC 指定アクセス
	uint32 read_8_fc(busaddr addr);
	uint32 read_16_fc(busaddr addr);
	uint32 read_32_fc(busaddr addr);
	void write_8_fc(busaddr addr, uint32 data);
	void write_16_fc(busaddr addr, uint32 data);
	void write_32_fc(busaddr addr, uint32 data);

	// 基本アクセスルーチンを使った便利サブルーチン
	inline void push_16(uint32 data) {
		reg.A[7] -= 2;
		write_16(reg.A[7], data);
	}
	inline void push_32(uint32 data) {
		reg.A[7] -= 4;
		write_32(reg.A[7], data);
	}
	inline uint32 pop_16() {
		uint32 data = read_16(reg.A[7]);
		reg.A[7] += 2;
		return data;
	}
	inline uint32 pop_32() {
		uint32 data = read_32(reg.A[7]);
		reg.A[7] += 4;
		return data;
	}

	// バスアクセス
	uint32 fetch_16_bus(uint32 addr);
	uint32 fetch_32_bus(uint32 addr);
	uint32 read_8_bus(busaddr addr);
	uint32 read_16_bus(busaddr addr);
	uint32 read_32_bus(busaddr addr);
	void write_8_bus(busaddr addr, uint32 data);
	void write_16_bus(busaddr addr, uint32 data);
	void write_32_bus(busaddr addr, uint32 data);
	bool fetch_burst_bus(uint32 addr, uint32 *dst);

#define OP_PROTO(name)	void __CONCAT(op_,name)()
#include "m68030ops.h"
#undef OP_PROTO

	void ops_divu_32_32(uint r, uint q);
	void ops_divu_64_32(uint r, uint q);
	void ops_divs_32_32(uint r, uint q);
	void ops_divs_64_32(uint r, uint q);
	void ops_movec_rc_rn();
	void ops_movec_rn_rc();
	void ops_movep();
	// もともとソース内のテンプレートだった
	void ops_movem_list_ea(int bytesize);
	void ops_movem_ea_list(int bytesize);
	void ops_trapcc(uint cond);
	inline void ops_bra();

	// 68030core.cpp
	void ops_rte();
	void ops_stop();
	// vm/mpu680x0.cpp
	void ops_reset();

#include "m68030fpu.h"
	void fpu_op_start();

	// キャッシュ
	void SetCACR(uint32);
	uint32 fetch_16_cache(uint32 addr);
	uint32 fetch_32_cache(uint32 addr);
	uint32 fetch_32_cache_aligned(uint32 addr);
#if 0
	uint32 read_8_cache(busaddr addr);
	uint32 read_16_cache(busaddr addr);
	uint32 read_32_cache(busaddr addr);
	void write_8_cache(busaddr addr, uint32 data);
	void write_16_cache(busaddr addr, uint32 data);
	void write_32_cache(busaddr addr, uint32 data);
#endif

	// MMU
	bool SetSRP(uint32, uint32);
	bool SetCRP(uint32, uint32);
	void SetTT(int idx, uint32);
	bool SetTC(uint32);
	void SetMMUSR(uint16);

	bool TranslateRead();
	bool TranslateWrite();

	bool CheckTT();
	uint32 get_fc();
	void mmu_op_illg2();
	void mmu_op_pflush();
	void mmu_op_pflush_ea();
	void mmu_op_pload();
	void mmu_op_ptest();
	// MMU 有効状態を更新。(TC と TT の変更で呼ばれる)
	void mmu_enable_changed();

	// 例外
	void GenerateExframe0(uint vector, uint32 pc, uint16 sr);
	void GenerateExframe2(uint vector, uint16 sr);
	void GenerateExframeB(uint vector, uint16 sr);
	int  ExceptionReset();
	void ExceptionBuserr(uint vector);
	uint64 ExceptionInterrupt();
	void ExceptionTrap15();
	void Exception(uint vector);
	void ExceptionGeneric(uint vector, uint32 inst);

	// 二重バスフォールト
	void DoubleBusFault(const char *where);

 public:
	// レジスタ
	m68kreg reg {};

	// MMU work
	m68030TT mmu_tt[2] /*{}*/;
	m68030TC mmu_tc /*{}*/;
	m68030ATC atc {this};
	bool mmu_enable {};		// MMU が有効なら true

	// バス
	m68kbus bus {};
	// 現在の特権状態でのプログラム空間とデータ空間アクセス用。
	busaddr fc_inst {};		// こっちは R 含む
	busaddr fc_data {};		// こっちは R/W 含まない

 private:
	uint16 ir {};
	uint16 ir2 {};
	// 現在実行中の命令の先頭アドレス
	uint32 ppc {};

	// レジスタ(状態保存用)
	uint32 save_pd[8] {};	// -(An) を保存
	uint32 save_pi[8] {};	// (An)+ を保存
	uint8  flag_pd {};		// -(An) のどのレジスタを保存したか
	uint8  flag_pi {};		// (An)+ のどのレジスタを保存したか

	// キャッシュ
	bool icache_enable {};
	bool icache_freeze {};
	std::unique_ptr<m68030Cache> icache /*{}*/;

	// FPU
	int has_fpu {};			// 0:FPUなし、1:68881、2:68882(?)
	bool fpu_dirty {};
	struct fpemu fe {};

	// CPU の実行状態
	uint32 cpu_state {};

	// 割り込みペンディング
	bool intr_pending {};

	// 現在の外部割り込みレベル
	int intr_level {};

	// 電源オンからの累積消費サイクル
	uint64 used_cycle {};

	// 今回のバスウェイト [nsec]
	uint64 buswait {};

	// 外部バスがバースト転送に対応していれば true
	bool has_burst_access {};

	int32 cycle_nocache {};		// ノーキャッシュケースの時の追加分

	// A-Line, F-Line 命令エミュレーションのコールバックとユーザ引数。
	// ユーザ引数には関与しないので呼び出し側が自由に指定してよい。
	// コールバック側で処理完了したら true を返す。
	// デフォルトの処理(すなわちm68k例外)にする場合は false を返す。
	bool (*aline_callback)(MPU680x0Device *, void *) {};
	void *aline_arg {};
	bool (*fline_callback)(MPU680x0Device *, void *) {};
	void *fline_arg {};

	// ブランチ履歴 (例外履歴を含む)
	BranchHistory_m680x0 brhist { OBJ_MPU_BRHIST };
	// 例外履歴のみ
	BranchHistory_m680x0 exhist { OBJ_MPU_EXHIST };

	// 割り込みイベント
	Event intr_event { this };

	// トレースでコールバックを差し替えることができるように
	// イベントコールバックを変数に入れてある
	EventCallback_t exec_short {};
	EventCallback_t exec_full {};

	// レジスタモニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateReg);
	Monitor reg_monitor { this };

	// ATC モニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateATC);
	Monitor atc_monitor { this };

	// キャッシュモニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateCache);
	Monitor cache_monitor { this };

	InterruptDevice *interruptdev {};
	VectorTable *vectortable {};
};

// これを呼びたい人は mpu を持ってるはず。
static inline MPU680x0Device *GetMPU680x0Device(Device *mpu_) {
	return dynamic_cast<MPU680x0Device*>(mpu_);
}

==> nono-1.0.0.tar.gz: nono-1.0.0/vm/mpu680x0.h <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU (MC68030)
//

#pragma once

#include "mpu.h"
#include "branchhistory.h"
#include "bus.h"
#include "event.h"
#include "m68030.h"
#include "m68030mmu.h"

class InterruptDevice;
class m68030Cache;
struct m68030CacheLine;

class MPU680x0Device : public MainMPUDevice
{
	using inherited = MainMPUDevice;
	friend class m68030MMU;

	// CPU の実行状態
	static const uint32 CPU_STATE_NORMAL	= 0;
	static const uint32 CPU_STATE_STOP		= 1;
	static const uint32 CPU_STATE_HALT		= 2;

 public:
	MPU680x0Device();
	~MPU680x0Device() override;

	void SetLogLevel(int loglevel_) override;
	bool Init() override;
	void ResetHard(bool poweron) override;

	// 現在の PPC を取得
	uint32 GetPPC() const override { return ppc; }

	// 現在の VBR を取得
	uint32 GetVBR() const override { return reg.vbr; }

	// MPU 名を取得
	const char *GetMPUName() const override { return "MC68030"; }

	bool IsSuper() const	{ return reg.s; }
	bool IsMaster() const	{ return reg.m; }

	// CPU の実行状態 (uint32) を返す。
	uint32 GetState() const	{ return cpu_state; }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override { return bus.laddr.Addr(); }
	uint32 GetPaddr() const override { return bus.paddr.Addr(); }

	// 割り込みレベルが変わったことを MPU に通知
	void Interrupt(int level) override;

	// A-Line, F-Line 命令エミュレーションのコールバックを指定する。
	// (Human68k エミュレーションで使用)
	void SetALineCallback(bool (*callback)(MPU680x0Device*, void*), void *arg);
	void SetFLineCallback(bool (*callback)(MPU680x0Device*, void*), void *arg);

	// FPU を持っているか
	bool HaveFPU() const { return has_fpu; }

	// バーストアクセスをサポートするか
	void EnableBurstAccess(bool enable) { has_burst_access = enable; }

	void SetCI() override {
		bus.ci = true;
	}

	// ホールト
	void Halt();

	// 基本アクセス。内部から使用する。
	// アドレスは論理アドレスで、空間は現在の命令/データ空間。
	// バスエラーなら C++ の例外をスローする。
	uint32 fetch_2();
	uint32 fetch_4();
	uint32 read_n(uint32 laddr, uint size) {
		busaddr addr =
			busaddr(laddr) | fc_data | BusAddr::R | busaddr::Size(size);
		return read_data(addr);
	}
	void write_n(uint32 laddr, uint size, uint32 data) {
		busaddr addr =
			busaddr(laddr) | fc_data | BusAddr::W | busaddr::Size(size);
		write_data(addr, data);
	}
	uint32 read_1(uint32 laddr) { return read_n(laddr, 1); }
	uint32 read_2(uint32 laddr) { return read_n(laddr, 2); }
	uint32 read_4(uint32 laddr) { return read_n(laddr, 4); }
	void write_1(uint32 laddr, uint32 data) { write_n(laddr, 1, data); }
	void write_2(uint32 laddr, uint32 data) { write_n(laddr, 2, data); }
	void write_4(uint32 laddr, uint32 data) { write_n(laddr, 4, data); }

	// MMU アクセス
	busaddr TranslatePeek(busaddr laddr);
	// 統計
	void CalcStat();

	// レジスタ
	uint16 GetIR() const	{ return ir; }
	uint16 GetIR2() const	{ return ir2; }
	// SR を変更する (Human68k から使う)
	void SetSR(uint16);
	uint16 GetSR() const	{ return reg.GetSR(); }

	uint64 GetSRP()  const { return reg.srp.q; }
	uint32 GetSRPh() const { return reg.srp.h; }
	uint32 GetSRPl() const { return reg.srp.l; }
	uint64 GetCRP()  const { return reg.crp.q; }
	uint32 GetCRPh() const { return reg.crp.h; }
	uint32 GetCRPl() const { return reg.crp.l; }
	uint64 GetTT(int idx) const { return reg.tt[idx]; }
	uint32 GetTC() const { return reg.tc; }
	uint16 GetMMUSR() const { return reg.mmusr; }

 private:
	// リセット例外コールバック
	void ResetCallback(Event& ev);

	// 命令実行系コールバック
	void ExecShort(Event& ev);
	void ExecFull(Event& ev);
	void ExecTrace(Event& ev);

	// トレース設定。デバッガから呼び出される。
	void SetTrace(bool trace_on) override;

	// 割り込みイベントコールバック
	void InterruptCallback(Event& ev);

	// 動作モード変更
	void ChangeState(uint32 new_state);

	// 次の実行サイクルを Full にする。
	void MakeNextFull();

	inline void CYCLE(int cc) {
		used_cycle += cc;
	}
	inline void CYCLE2(int cc, int ncc) {
		used_cycle += cc;
		cycle_nocache += ncc - cc;
	}

	// PC を addr に変更する。
	// ベクタへのジャンプはここではなく JumpVector()。
	inline void Jump(uint32 addr) {
		// 先にブランチ履歴に記録
		brhist.AddEntry(ppc | (IsSuper() ? 1 : 0), addr, ir);

		if (__predict_false((addr & 1) != 0)) {
			ExceptionBuserr(M68K::EXCEP_ADDRERR);
			return;
		}
		reg.pc = addr;
	}

	// -(An) 実行前にレジスタを保存する。
	// バスエラーからの復帰用。
	inline void save_reg_pd(int n) {
		// 該当ビットのフラグを立てる
		flag_pd |= (1 << n);

		// 保存
		save_pd[n] = reg.A[n];
	}

	// (An)+ 実行前にレジスタを保存する。
	// バスエラーからの復帰用。
	inline void save_reg_pi(int n) {
		// 該当ビットのフラグを立てる
		flag_pi |= (1 << n);

		// 保存
		save_pi[n] = reg.A[n];
	}

	// -(An) 実行前にレジスタを(まだ保存してなければ)保存する。
	// 1命令で同じレジスタを2回 -(An) する可能性がある命令用。
	inline void save_reg_pd_if(int n) {
		if ((flag_pd & (1 << n)) == 0) {
			save_reg_pd(n);
		}
	}
	// (An)+ 実行前にレジスタを(まだ保存してなければ)保存する。
	// 1命令で同じレジスタを2回 (An)+ する可能性がある命令用。
	inline void save_reg_pi_if(int n) {
		if ((flag_pi & (1 << n)) == 0) {
			save_reg_pi(n);
		}
	}

	void restore_reg_pi();
	void restore_reg_pd();
	void JumpVector(uint vector, bool from_buserr);

 protected:
#include "m68030ea.h"

 private:
	// 基本アクセスルーチンを使った便利サブルーチン
	inline void push_2(uint32 data) {
		reg.A[7] -= 2;
		write_2(reg.A[7], data);
	}
	inline void push_4(uint32 data) {
		reg.A[7] -= 4;
		write_4(reg.A[7], data);
	}
	inline uint32 pop_2() {
		uint32 data = read_2(reg.A[7]);
		reg.A[7] += 2;
		return data;
	}
	inline uint32 pop_4() {
		uint32 data = read_4(reg.A[7]);
		reg.A[7] += 4;
		return data;
	}

	// バスアクセス
	uint32 read_data(busaddr addr);
	busdata read_nocache(busaddr addr);
	busdata read_cache(m68030Cache *cache, busaddr addr);
	busdata read_line(m68030CacheLine *, uint32 tag, uint32 opermask);
	busdata read_single(busdata bd, uint32 readmask, uint32 opermask);

	void write_data(busaddr addr, uint32 data);
	busdata write_bus(busaddr addr, uint32 data);

	busdata read_phys();
	busdata write_phys(uint32 data);
	bool read_phys_burst(uint32 *dst);

#define OP_PROTO(name)	void __CONCAT(op_,name)()
#include "m68030ops.h"
#undef OP_PROTO

	void ops_divu_32_32(uint r, uint q);
	void ops_divu_64_32(uint r, uint q);
	void ops_divs_32_32(uint r, uint q);
	void ops_divs_64_32(uint r, uint q);
	void ops_movec_rc_rn();
	void ops_movec_rn_rc();
	void ops_movep();
	// もともとソース内のテンプレートだった
	void ops_movem_list_ea(int bytesize);
	void ops_movem_ea_list(int bytesize);
	void ops_trapcc(uint cond);
	inline void ops_bra();

	// 68030core.cpp
	void ops_rte();
	void ops_stop();
	// vm/mpu680x0.cpp
	void ops_reset();

#include "m68030fpu.h"
	void fpu_op_start();

	// キャッシュ
	void SetCACR(uint32);

	// MMU
	bool SetSRP(uint32, uint32);
	bool SetCRP(uint32, uint32);
	void SetTT(int idx, uint32);
	bool SetTC(uint32);
	void SetMMUSR(uint16);

	bool TranslateRead();
	bool TranslateWrite();

	bool CheckTT();
	uint32 get_fc();
	void mmu_op_illg2();
	void mmu_op_pflush();
	void mmu_op_pflush_ea();
	void mmu_op_pload();
	void mmu_op_ptest();
	// MMU 有効状態を更新。(TC と TT の変更で呼ばれる)
	void mmu_enable_changed();

	// 例外
	void GenerateExframe0(uint vector, uint32 pc, uint16 sr);
	void GenerateExframe2(uint vector, uint16 sr);
	void GenerateExframeB(uint vector, uint16 sr);
	int  ExceptionReset();
	void ExceptionBuserr(uint vector);
	uint64 ExceptionInterrupt();
	void ExceptionTrap15();
	void Exception(uint vector);
	void ExceptionGeneric(uint vector, uint32 inst);

	// 二重バスフォールト
	void DoubleBusFault(const char *where);

 public:
	// レジスタ
	m68kreg reg {};

	// MMU work
	m68030TT mmu_tt[2] /*{}*/;
	m68030TC mmu_tc /*{}*/;
	m68030ATC atc {this};
	bool mmu_enable {};		// MMU が有効なら true

	// バス
	m68kbus bus {};
	// 現在の特権状態でのプログラム空間とデータ空間アクセス用。
	busaddr fc_inst {};		// こっちは R 含む
	busaddr fc_data {};		// こっちは R/W 含まない

 private:
	uint16 ir {};
	uint16 ir2 {};
	// 現在実行中の命令の先頭アドレス
	uint32 ppc {};

	// レジスタ(状態保存用)
	uint32 save_pd[8] {};	// -(An) を保存
	uint32 save_pi[8] {};	// (An)+ を保存
	uint8  flag_pd {};		// -(An) のどのレジスタを保存したか
	uint8  flag_pi {};		// (An)+ のどのレジスタを保存したか

	// キャッシュ
	std::unique_ptr<m68030Cache> icache /*{}*/;
	std::unique_ptr<m68030Cache> dcache /*{}*/;

	// FPU
	int has_fpu {};			// 0:FPUなし、1:68881、2:68882(?)
	bool fpu_dirty {};
	struct fpemu fe {};

	// CPU の実行状態
	uint32 cpu_state {};

	// 割り込みペンディング
	bool intr_pending {};

	// 現在の外部割り込みレベル
	int intr_level {};

	// 電源オンからの累積消費サイクル
	uint64 used_cycle {};

	// 今回のバスウェイト [nsec]
	uint64 buswait {};

	// 外部バスがバースト転送に対応していれば true
	bool has_burst_access {};

	int32 cycle_nocache {};		// ノーキャッシュケースの時の追加分

	// A-Line, F-Line 命令エミュレーションのコールバックとユーザ引数。
	// ユーザ引数には関与しないので呼び出し側が自由に指定してよい。
	// コールバック側で処理完了したら true を返す。
	// デフォルトの処理(すなわちm68k例外)にする場合は false を返す。
	bool (*aline_callback)(MPU680x0Device *, void *) {};
	void *aline_arg {};
	bool (*fline_callback)(MPU680x0Device *, void *) {};
	void *fline_arg {};

	// ブランチ履歴 (例外履歴を含む)
	BranchHistory_m680x0 brhist { OBJ_MPU_BRHIST };
	// 例外履歴のみ
	BranchHistory_m680x0 exhist { OBJ_MPU_EXHIST };

	// 割り込みイベント
	Event intr_event { this };

	// トレースでコールバックを差し替えることができるように
	// イベントコールバックを変数に入れてある
	EventCallback_t exec_short {};
	EventCallback_t exec_full {};

	// レジスタモニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateReg);
	void MonitorUpdateFPU(TextScreen&, int y, m68kreg&);
	Monitor reg_monitor { this };

	// ATC モニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateATC);
	Monitor atc_monitor { this };

	// キャッシュモニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdateCache);
	int MonitorUpdateCache1(TextScreen&, int y, m68030Cache *,
		char (*)(const m68030CacheLine&));
	Monitor cache_monitor { this };

	InterruptDevice *interruptdev {};
	VectorTable *vectortable {};
};

// これを呼びたい人は mpu を持ってるはず。
static inline MPU680x0Device *GetMPU680x0Device(Device *mpu_) {
	return dynamic_cast<MPU680x0Device*>(mpu_);
}
